# -*- coding: ISO-8859-1 -*-
# Copyright (C) 2004-2005 Juan David Ibez Palomar <jdavid@itaapy.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Import from the Standard Library
import time
import warnings

# Import from itools
from itools.handlers.Folder import Folder

# Import from iKaaro
from exceptions import UserError
from Handler import Handler
from utils import comeback



class VersioningAware(object):

    def commit_revision(self):
        # Add folders to archived file (XXX)
        archive_id = self.get_property('id')
        if archive_id is None:
            # XXX Better to be logged
            warnings.warn('Metadata lacks the property "id"')
        else:
            archive = self.get_root().get_handler('.archive')
            archive_id = self.get_archive_id()
            path = archive_id.split('/')
            for i in range(len(path)):
                x = '/'.join(path[:i+1])
                if not archive.resource.has_resource(x):
                    archive.set_handler(x, Folder())
            # Store revision in the archive
            revisions = archive.get_handler(archive_id)
            seconds_since_the_epoch = time.mktime(self.mtime.timetuple())
            revision_name = str(seconds_since_the_epoch)
            while revisions.resource.has_resource(revision_name):
                seconds_since_the_epoch += 0.1
                revision_name = str(seconds_since_the_epoch)
            revisions.set_handler(revision_name, self)


    #########################################################################
    # User Interface
    #########################################################################
    def get_views(self):
        return ['history_form']


    #######################################################################
    # History
    history_form__access__ = Handler.is_allowed_to_edit
    history_form__label__ = u'History'
    def history_form(self):
        namespace = {}
        # Revisions
        root = self.get_root()
        archive_id = self.get_archive_id()
        archive = root.get_handler('.archive').get_handler(archive_id)
        revisions = archive.resource.get_resource_names()
        revisions.sort()
        revisions = [ (name, archive.resource.get_resource(name))
                      for name in revisions ]
        revisions = [{'name': name,
                      'date': revision.get_mtime().strftime('%Y/%m/%d'),
                      'username': '', # XXX revision.username,
                      'action': '-',
                      'size': revision.get_size()}
                     for name, revision in revisions ]
        revisions.reverse()
        namespace['revisions'] = revisions
        # Compare
        namespace['compare'] = hasattr(self, 'compare')

        handler = self.get_handler('/ui/File_history.xml')
        return handler.stl(namespace)


    copy_to_present__access__ = Handler.is_allowed_to_edit
    def copy_to_present(self, names=[], **kw):
        if len(names) != 1:
            raise UserError, \
                  self.gettext('You must select one and only one revision.')

        archive = self.get_root().get_handler('.archive')
        revisions = archive.get_handler(self.get_archive_id())
        revision = revisions.get_handler(names[0])
        self.load_state(revision.resource)

        message = self.gettext('Revision copied to present.')
        comeback('history_form', message)
