# -*- coding: ISO-8859-1 -*-
# Copyright (C) 2002-2005 Juan David Ibez Palomar <jdavid@itaapy.com>
#               2005 Nicolas Oyez <noyez@itaapy.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Import from the Standard Library
import logging
import logging.config
import os
import thread

# Import from itools
from itools import uri, get_abspath
from itools.resources import zope2
from itools.gettext import domains
from itools.zope import get_context

# Import from iKaaro
from Root import Root
from Folder import Folder
import Document
import File
import csv
import flash
import html
import images
import msoffice
import openoffice
import pdf
import text


def initialize(context):
    # XXX The product path, it maybe useful (i.e. to fix the localization
    # problems)
##    print context._ProductContext__pack.__path__[0]
    Root.register_in_zope(context)


#############################################################################
# Logging facilities
#############################################################################
filename = get_abspath(globals(), 'logging.conf')
logging.config.fileConfig(filename)


###########################################################################
# Register
###########################################################################
Folder.register_document_type(Folder)
Folder.register_document_type(File.File)
Folder.register_document_type(text.Text)
Folder.register_document_type(Document.HTML)

# Register domain (i18n)
domains.register_domain('ikaaro', get_abspath(globals(), 'locale'))


###########################################################################
# Check for required software
###########################################################################
cmds = ['xlhtml', 'pdftotext', 'catdoc', 'ppthtml', 'iconv', 'links',
        'unzip', 'pdftohtml', 'wvHtml']

paths = os.getenv('PATH').split(':')
all_names = set()
for path in paths:
    path = path.strip()
    try:
        names = os.listdir(path)
    except OSError:
        pass
    else:
        all_names = all_names.union(names)

for name in cmds:
    if name not in all_names:
        print 'You need to install the command "%s".' % name



###########################################################################
# Zope hook to get control of the publishing process
###########################################################################

roots = {}
roots_lock = thread.allocate_lock()



def __before_publishing_traverse__(self, REQUEST):
    # XXX Catch and log exceptions (zope ignores them so bugs could be
    # missed)

    # Parse the path to be traversed
    path = REQUEST['TraversalRequestNameStack'][:]
    path.reverse()
    path.insert(0, self.getId())
    path = uri.Path(path)

    context = get_context()
    if path[-1].name.startswith('manage_'):
        context.zmi = True
    else:
        context.zmi = False
        # Set the root resource
        root_path = '/'.join(self.getPhysicalPath())
        context.root_resource = zope2.get_resource(root_path)
        # Set the path and view variables
        if path[-1].name:
            context.path = path
            context.method = None
        else:
            context.path = path[:-1]
            context.method = path[-1].param
        # Cut the Zope publishing process
        REQUEST['TraversalRequestNameStack'] = []


def get_root(root_resource):
    key = (thread.get_ident(), root_resource._path)
    if key in roots:
        root = roots[key]
        if root.is_outdated():
            root.load_state()
    else:
        from Metadata import Metadata
        # Get the format
        metadata_resource = root_resource.get_resource('.metadata')
        metadata = Metadata(metadata_resource)
        format = metadata.get_property('format')
        # Build the handler
        handler_class = Root.metatypes_registry[format]
        root = handler_class(root_resource)
        # Store in the cache
        roots_lock.acquire()
        try:
            roots[key] = root
        finally:
            roots_lock.release()

    return root


def __browser_default__(request):
    context = get_context()
    request, response = context.request, context.response

    path = context.path
    root_resource = context.root_resource

    # Required by Epoz
    # XXX To be removed once we move to Kupu
    if len(path) > 1:
        if len([ x for x in path if x.name == 'misc_' ]) > 0:
            i = [ x.name for x in path ].index('misc_')
            path = path[i+1:]
            x = request.zope_request['PARENTS'][-1].misc_
            while path:
                segment = path.pop(0)
                try:
                    x = getattr(x, segment.name)
                except AttributeError:
                    x = x[segment.name]
            return x, ['index_html']
        if path[-1].name == 'epoz_blank_iframe.html':
            # XXX Does this work?
            return root_resource, ['epoz_blank_iframe.html']

    # Build the handler for the application's root
    root = get_root(root_resource)

    root.publish()
    return response, ['get_body']


